@@ -17,7 +17,7 @@ use crate::typed_ast::{
 };
 use crate::unit::{CanonicalName, Unit};
 use crate::unit_registry::{UnitMetadata, UnitRegistry};
-use crate::value::FunctionReference;
+use crate::value::{FunctionReference, Value};
 use crate::vm::{Constant, ExecutionContext, Op, Vm};
 use crate::{decorator, ffi, Type};
 
@@ -69,15 +69,15 @@ impl BytecodeInterpreter {
                     .rposition(|l| &l.identifier == identifier)
                 {
                     self.vm.add_op1(Op::GetUpvalue, upvalue_position as u16);
-                } else if LAST_RESULT_IDENTIFIERS.contains(&identifier.as_str()) {
+                } else if LAST_RESULT_IDENTIFIERS.contains(identifier) {
                     self.vm.add_op(Op::GetLastResult);
-                } else if let Some(is_foreign) = self.functions.get(identifier) {
+                } else if let Some(is_foreign) = self.functions.get(*identifier) {
                     let index = self
                         .vm
                         .add_constant(Constant::FunctionReference(if *is_foreign {
-                            FunctionReference::Foreign(identifier.clone())
+                            FunctionReference::Foreign(identifier.to_string())
                         } else {
-                            FunctionReference::Normal(identifier.clone())
+                            FunctionReference::Normal(identifier.to_string())
                         }));
                     self.vm.add_op1(Op::LoadConstant, index);
                 } else {
@@ -159,7 +159,7 @@ impl BytecodeInterpreter {
             Expression::FunctionCall(_span, _full_span, name, args, _type) => {
                 // Put all arguments on top of the stack
                 for arg in args {
-                    self.compile_expression_with_simplify(arg)?;
+                    self.compile_expression(arg)?;
                 }
 
                 if let Some(idx) = self.vm.get_ffi_callable_idx(name) {
@@ -178,10 +178,10 @@ impl BytecodeInterpreter {
 
                 let sorted_exprs = exprs
                     .iter()
-                    .sorted_by_key(|(n, _)| struct_info.fields.get_index_of(n).unwrap());
+                    .sorted_by_key(|(n, _)| struct_info.fields.get_index_of(*n).unwrap());
 
                 for (_, expr) in sorted_exprs.rev() {
-                    self.compile_expression_with_simplify(expr)?;
+                    self.compile_expression(expr)?;
                 }
 
                 let struct_info_idx = self.vm.get_structinfo_idx(&struct_info.name).unwrap() as u16;
@@ -190,7 +190,7 @@ impl BytecodeInterpreter {
                     .add_op2(Op::BuildStructInstance, struct_info_idx, exprs.len() as u16);
             }
             Expression::AccessField(_span, _full_span, expr, attr, struct_type, _result_type) => {
-                self.compile_expression_with_simplify(expr)?;
+                self.compile_expression(expr)?;
 
                 let Type::Struct(ref struct_info) = struct_type.to_concrete_type() else {
                     unreachable!(
@@ -198,14 +198,14 @@ impl BytecodeInterpreter {
                     );
                 };
 
-                let idx = struct_info.fields.get_index_of(attr).unwrap();
+                let idx = struct_info.fields.get_index_of(*attr).unwrap();
 
                 self.vm.add_op1(Op::AccessStructField, idx as u16);
             }
             Expression::CallableCall(_span, callable, args, _type) => {
                 // Put all arguments on top of the stack
                 for arg in args {
-                    self.compile_expression_with_simplify(arg)?;
+                    self.compile_expression(arg)?;
                 }
 
                 // Put the callable on top of the stack
@@ -221,7 +221,7 @@ impl BytecodeInterpreter {
                 for part in string_parts {
                     match part {
                         StringPart::Fixed(s) => {
-                            let index = self.vm.add_constant(Constant::String(s.clone()));
+                            let index = self.vm.add_constant(Constant::String(s.to_string()));
                             self.vm.add_op1(Op::LoadConstant, index)
                         }
                         StringPart::Interpolation {
@@ -229,9 +229,9 @@ impl BytecodeInterpreter {
                             span: _,
                             format_specifiers,
                         } => {
-                            self.compile_expression_with_simplify(expr)?;
+                            self.compile_expression(expr)?;
                             let index = self.vm.add_constant(Constant::FormatSpecifiers(
-                                format_specifiers.clone(),
+                                format_specifiers.map(|s| s.to_string()),
                             ));
                             self.vm.add_op1(Op::LoadConstant, index)
                         }
@@ -263,7 +263,7 @@ impl BytecodeInterpreter {
             }
             Expression::List(_, elements, _) => {
                 for element in elements {
-                    self.compile_expression_with_simplify(element)?;
+                    self.compile_expression(element)?;
                 }
 
                 self.vm.add_op1(Op::BuildList, elements.len() as u16);
@@ -276,32 +276,6 @@ impl BytecodeInterpreter {
         Ok(())
     }
 
-    fn compile_expression_with_simplify(&mut self, expr: &Expression) -> Result<()> {
-        self.compile_expression(expr)?;
-
-        match expr {
-            Expression::Scalar(..)
-            | Expression::Identifier(..)
-            | Expression::UnitIdentifier(..)
-            | Expression::FunctionCall(..)
-            | Expression::CallableCall(..)
-            | Expression::UnaryOperator(..)
-            | Expression::BinaryOperator(_, BinaryOperator::ConvertTo, _, _, _)
-            | Expression::Boolean(..)
-            | Expression::String(..)
-            | Expression::Condition(..)
-            | Expression::InstantiateStruct(..)
-            | Expression::AccessField(..)
-            | Expression::List(..) => {}
-            Expression::BinaryOperator(..) | Expression::BinaryOperatorForDate(..) => {
-                self.vm.add_op(Op::FullSimplify);
-            }
-            Expression::TypedHole(_, _) => unreachable!("Typed holes cause type inference errors"),
-        }
-
-        Ok(())
-    }
-
     fn compile_define_variable(&mut self, define_variable: &DefineVariable) -> Result<()> {
         let DefineVariable(identifier, decorators, expr, _annotation, _type, _readable_type) =
             define_variable;
@@ -309,18 +283,17 @@ impl BytecodeInterpreter {
 
         // For variables, we ignore the prefix info and only use the names
         let aliases = crate::decorator::name_and_aliases(identifier, decorators)
-            .map(|(name, _)| name)
-            .cloned()
+            .map(|(name, _)| name.to_owned())
             .collect::<Vec<_>>();
         let metadata = LocalMetadata {
-            name: crate::decorator::name(decorators),
-            url: crate::decorator::url(decorators),
+            name: crate::decorator::name(decorators).map(ToOwned::to_owned),
+            url: crate::decorator::url(decorators).map(ToOwned::to_owned),
             description: crate::decorator::description(decorators),
             aliases: aliases.clone(),
         };
 
         for alias_name in aliases {
-            self.compile_expression_with_simplify(expr)?;
+            self.compile_expression(expr)?;
 
             self.locals[current_depth].push(Local {
                 identifier: alias_name.clone(),
@@ -338,7 +311,7 @@ impl BytecodeInterpreter {
     ) -> Result<()> {
         match stmt {
             Statement::Expression(expr) => {
-                self.compile_expression_with_simplify(expr)?;
+                self.compile_expression(expr)?;
                 self.vm.add_op(Op::Return);
             }
             Statement::DefineVariable(define_variable) => {
@@ -351,7 +324,7 @@ impl BytecodeInterpreter {
                 parameters,
                 Some(expr),
                 local_variables,
-                _return_type,
+                _function_type,
                 _return_type_annotation,
                 _readable_return_type,
             ) => {
@@ -362,7 +335,7 @@ impl BytecodeInterpreter {
                 let current_depth = self.current_depth();
                 for parameter in parameters {
                     self.locals[current_depth].push(Local {
-                        identifier: parameter.1.clone(),
+                        identifier: parameter.1.to_string(),
                         depth: current_depth,
                         metadata: LocalMetadata::default(),
                     });
@@ -371,14 +344,15 @@ impl BytecodeInterpreter {
                     self.compile_define_variable(local_variables)?;
                 }
 
-                self.compile_expression_with_simplify(expr)?;
+                self.compile_expression(expr)?;
+
                 self.vm.add_op(Op::Return);
 
                 self.locals.pop();
 
                 self.vm.end_function();
 
-                self.functions.insert(name.clone(), false);
+                self.functions.insert(name.to_string(), false);
             }
             Statement::DefineFunction(
                 name,
@@ -397,7 +371,7 @@ impl BytecodeInterpreter {
                 self.vm
                     .add_foreign_function(name, parameters.len()..=parameters.len());
 
-                self.functions.insert(name.clone(), true);
+                self.functions.insert(name.to_string(), true);
             }
             Statement::DefineDimension(_name, _dexprs) => {
                 // Declaring a dimension is like introducing a new type. The information
@@ -405,7 +379,7 @@ impl BytecodeInterpreter {
             }
             Statement::DefineBaseUnit(unit_name, decorators, annotation, type_) => {
                 let aliases = decorator::name_and_aliases(unit_name, decorators)
-                    .map(|(name, ap)| (name.clone(), ap))
+                    .map(|(name, ap)| (name.to_owned(), ap))
                     .collect();
 
                 self.vm
@@ -417,13 +391,13 @@ impl BytecodeInterpreter {
                             readable_type: annotation
                                 .as_ref()
                                 .map(|a| a.pretty_print())
-                                .unwrap_or(type_.to_readable_type(dimension_registry)),
+                                .unwrap_or(type_.to_readable_type(dimension_registry, false)),
                             aliases,
-                            name: decorator::name(decorators),
+                            name: decorator::name(decorators).map(ToOwned::to_owned),
                             canonical_name: decorator::get_canonical_unit_name(
                                 unit_name, decorators,
                             ),
-                            url: decorator::url(decorators),
+                            url: decorator::url(decorators).map(ToOwned::to_owned),
                             description: decorator::description(decorators),
                             binary_prefixes: decorators.contains(&Decorator::BinaryPrefixes),
                             metric_prefixes: decorators.contains(&Decorator::MetricPrefixes),
@@ -433,7 +407,7 @@ impl BytecodeInterpreter {
 
                 let constant_idx = self.vm.add_constant(Constant::Unit(Unit::new_base(
                     unit_name,
-                    crate::decorator::get_canonical_unit_name(unit_name.as_str(), &decorators[..]),
+                    crate::decorator::get_canonical_unit_name(unit_name, &decorators[..]),
                 )));
                 for (name, _) in decorator::name_and_aliases(unit_name, decorators) {
                     self.unit_name_to_constant_index
@@ -449,7 +423,7 @@ impl BytecodeInterpreter {
                 _readable_type,
             ) => {
                 let aliases = decorator::name_and_aliases(unit_name, decorators)
-                    .map(|(name, ap)| (name.clone(), ap))
+                    .map(|(name, ap)| (name.to_owned(), ap))
                     .collect();
 
                 let constant_idx = self.vm.add_constant(Constant::Unit(Unit::new_base(
@@ -462,29 +436,25 @@ impl BytecodeInterpreter {
                 let unit_information_idx = self.vm.add_unit_information(
                     unit_name,
                     Some(
-                        &crate::decorator::get_canonical_unit_name(
-                            unit_name.as_str(),
-                            &decorators[..],
-                        )
-                        .name,
+                        &crate::decorator::get_canonical_unit_name(unit_name, &decorators[..]).name,
                     ),
                     UnitMetadata {
                         type_: type_.to_concrete_type(), // We guarantee that derived-unit definitions do not contain generics, so no TGen(..)s can escape
                         readable_type: annotation
                             .as_ref()
                             .map(|a| a.pretty_print())
-                            .unwrap_or(type_.to_readable_type(dimension_registry)),
+                            .unwrap_or(type_.to_readable_type(dimension_registry, false)),
                         aliases,
-                        name: decorator::name(decorators),
+                        name: decorator::name(decorators).map(ToOwned::to_owned),
                         canonical_name: decorator::get_canonical_unit_name(unit_name, decorators),
-                        url: decorator::url(decorators),
+                        url: decorator::url(decorators).map(ToOwned::to_owned),
                         description: decorator::description(decorators),
                         binary_prefixes: decorators.contains(&Decorator::BinaryPrefixes),
                         metric_prefixes: decorators.contains(&Decorator::MetricPrefixes),
                     },
                 ); // TODO: there is some asymmetry here because we do not introduce identifiers for base units
 
-                self.compile_expression_with_simplify(expr)?;
+                self.compile_expression(expr)?;
                 self.vm
                     .add_op2(Op::SetUnitConstant, unit_information_idx, constant_idx);
 
@@ -507,7 +477,7 @@ impl BytecodeInterpreter {
             Statement::ProcedureCall(kind, args) => {
                 // Put all arguments on top of the stack
                 for arg in args {
-                    self.compile_expression_with_simplify(arg)?;
+                    self.compile_expression(arg)?;
                 }
 
                 let name = &ffi::procedures().get(kind).unwrap().name;
@@ -542,6 +512,13 @@ impl BytecodeInterpreter {
 
         let result = self.vm.run(&mut ctx);
 
+        let result = match result {
+            Ok(InterpreterResult::Value(Value::Quantity(q))) => {
+                Ok(InterpreterResult::Value(Value::Quantity(q.full_simplify())))
+            }
+            r => r,
+        };
+
         self.vm.debug();
 
         result
