@@ -5,6 +5,7 @@ mod lists;
 mod lookup;
 mod macros;
 mod math;
+mod plot;
 mod procedures;
 mod strings;
 
@@ -31,7 +32,7 @@ pub(crate) enum Callable {
 }
 
 pub(crate) struct ForeignFunction {
-    pub(crate) name: String,
+    pub(crate) name: &'static str,
     pub(crate) arity: ArityRange,
     pub(crate) callable: Callable,
 }
