@@ -6,6 +6,7 @@ use crate::unit::{is_multiple_of, Unit, UnitFactor};
 use itertools::Itertools;
 use num_rational::Ratio;
 use num_traits::{FromPrimitive, Zero};
+use pretty_dtoa::FmtFloatConfig;
 use thiserror::Error;
 
 #[derive(Clone, Debug, Error, PartialEq, Eq)]
@@ -23,20 +24,31 @@ pub type Result<T> = std::result::Result<T, QuantityError>;
 pub struct Quantity {
     value: Number,
     unit: Unit,
+    can_simplify: bool,
 }
 
 impl Quantity {
     pub fn new(value: Number, unit: Unit) -> Self {
-        Quantity { value, unit }
+        Quantity {
+            value,
+            unit,
+            can_simplify: true,
+        }
     }
 
     pub fn new_f64(value: f64, unit: Unit) -> Self {
         Quantity {
             value: Number::from_f64(value),
             unit,
+            can_simplify: true,
         }
     }
 
+    pub fn no_simplify(mut self) -> Self {
+        self.can_simplify = false;
+        self
+    }
+
     pub fn from_scalar(value: f64) -> Quantity {
         Quantity::new_f64(value, Unit::scalar())
     }
@@ -53,6 +65,10 @@ impl Quantity {
         self.value.to_f64() == 0.0
     }
 
+    pub fn abs(self) -> Self {
+        Quantity::new(self.value.abs(), self.unit)
+    }
+
     pub fn to_base_unit_representation(&self) -> Quantity {
         let (unit, factor) = self.unit.to_base_unit_representation();
         Quantity::new(self.value * factor, unit)
@@ -125,6 +141,10 @@ impl Quantity {
     }
 
     pub fn full_simplify(&self) -> Self {
+        if !self.can_simplify {
+            return self.clone();
+        }
+
         // Heuristic 1
         if let Ok(scalar_result) = self.convert_to(&Unit::scalar()) {
             return scalar_result;
@@ -172,7 +192,8 @@ impl Quantity {
             let group_representative = group_as_unit
                 .iter()
                 .max_by(|&f1, &f2| {
-                    // TODO: describe this heuristic
+                    // prefer base units over non-base. if multiple base units, prefer
+                    // those with a larger exponent
                     (f1.unit_id.is_base().cmp(&f2.unit_id.is_base()))
                         .then(f1.exponent.cmp(&f2.exponent))
                 })
@@ -251,10 +272,10 @@ impl std::ops::Add for &Quantity {
         } else if rhs.is_zero() {
             Ok(self.clone())
         } else {
-            Ok(Quantity {
-                value: self.value + rhs.convert_to(&self.unit)?.value,
-                unit: self.unit.clone(),
-            })
+            Ok(Quantity::new(
+                self.value + rhs.convert_to(&self.unit)?.value,
+                self.unit.clone(),
+            ))
         }
     }
 }
@@ -268,10 +289,10 @@ impl std::ops::Sub for &Quantity {
         } else if rhs.is_zero() {
             Ok(self.clone())
         } else {
-            Ok(Quantity {
-                value: self.value - rhs.convert_to(&self.unit)?.value,
-                unit: self.unit.clone(),
-            })
+            Ok(Quantity::new(
+                self.value - rhs.convert_to(&self.unit)?.value,
+                self.unit.clone(),
+            ))
         }
     }
 }
@@ -280,10 +301,7 @@ impl std::ops::Mul for Quantity {
     type Output = Quantity;
 
     fn mul(self, rhs: Self) -> Self::Output {
-        Quantity {
-            value: self.value * rhs.value,
-            unit: self.unit * rhs.unit,
-        }
+        Quantity::new(self.value * rhs.value, self.unit * rhs.unit)
     }
 }
 
@@ -291,10 +309,7 @@ impl std::ops::Div for Quantity {
     type Output = Quantity;
 
     fn div(self, rhs: Self) -> Self::Output {
-        Quantity {
-            value: self.value / rhs.value,
-            unit: self.unit / rhs.unit,
-        }
+        Quantity::new(self.value / rhs.value, self.unit / rhs.unit)
     }
 }
 
@@ -302,10 +317,7 @@ impl std::ops::Neg for Quantity {
     type Output = Quantity;
 
     fn neg(self) -> Self::Output {
-        Quantity {
-            value: -self.value,
-            unit: self.unit,
-        }
+        Quantity::new(-self.value, self.unit)
     }
 }
 
@@ -330,9 +342,17 @@ impl PartialOrd for Quantity {
 
 impl PrettyPrint for Quantity {
     fn pretty_print(&self) -> crate::markup::Markup {
+        self.pretty_print_with_options(None)
+    }
+}
+
+impl Quantity {
+    /// Pretty prints with the given options.
+    /// If options is None, default options will be used.
+    fn pretty_print_with_options(&self, options: Option<FmtFloatConfig>) -> crate::markup::Markup {
         use crate::markup;
 
-        let formatted_number = self.unsafe_value().pretty_print();
+        let formatted_number = self.unsafe_value().pretty_print_with_options(options);
 
         let unit_str = format!("{}", self.unit());
 
@@ -344,6 +364,22 @@ impl PrettyPrint for Quantity {
             }
             + markup::unit(unit_str)
     }
+
+    /// Pretty prints with the given precision. Disables e (scientific) notation.
+    /// Prints without fractional part if precision is 0.
+    pub fn pretty_print_with_precision(&self, precision: i8) -> crate::markup::Markup {
+        let options = FmtFloatConfig::default()
+            .min_decimal_digits(precision)
+            .max_decimal_digits(precision)
+            .add_point_zero(false)
+            .force_no_e_notation()
+            .round();
+        self.pretty_print_with_options(Some(options))
+    }
+
+    pub fn unsafe_value_as_string(&self) -> String {
+        self.unsafe_value().to_string()
+    }
 }
 
 impl std::fmt::Display for Quantity {
@@ -445,6 +481,24 @@ mod tests {
         }
     }
 
+    #[test]
+    fn abs() {
+        assert_eq!(
+            Quantity::new_f64(0.0, Unit::scalar()).abs(),
+            Quantity::new_f64(0.0, Unit::scalar())
+        );
+
+        assert_eq!(
+            Quantity::new_f64(1.0, Unit::scalar()).abs(),
+            Quantity::new_f64(1.0, Unit::scalar())
+        );
+
+        assert_eq!(
+            Quantity::new_f64(-1.0, Unit::scalar()).abs(),
+            Quantity::new_f64(1.0, Unit::scalar())
+        );
+    }
+
     #[test]
     fn full_simplify_basic() {
         let q = Quantity::new_f64(2.0, Unit::meter() / Unit::second());
