@@ -13,11 +13,12 @@ pub fn get_test_context_without_prelude() -> Context {
     let mut importer = FileSystemImporter::default();
     importer.add_path(module_path);
 
+    Context::use_test_exchange_rates();
     Context::new(importer)
 }
 
 pub fn get_test_context() -> Context {
-    static CONTEXT: Lazy<Result<Context, NumbatError>> = Lazy::new(|| {
+    static CONTEXT: Lazy<Result<Context, Box<NumbatError>>> = Lazy::new(|| {
         let mut context = get_test_context_without_prelude();
 
         let _ = context.interpret("use prelude", CodeSource::Internal)?;
